part of spark_modal;

typedef SparkPickerBuilder = Widget Function(BuildContext context, int index);

typedef SparkPickerChanged = void Function(int index);

typedef SparkPickerTitleBarBuilder = Widget Function(
    BuildContext context, Function cacel, Function confirm);

Future<T?> showSparkPicker<T>({
  required BuildContext context,
  required List<SparkPickerData<T>> options,
  double itemHeight = 40,
  SparkPickerBuilder? itemBuilder,
  bool showBar = true,
  String cancelText = '取消',
  String confirmText = '确定',
  TextStyle? cancelStyle,
  TextStyle? confirmStyle,
  double height = 300,
  SparkPickerTitleBarBuilder? barBuilder,
}) {
  return showSparkModalSheet<T>(
    context: context,
    shape: _defaultPickerShape,
    builder: (BuildContext _ctx) {
      return _SparkCommonPicker(
        dialogContext: _ctx,
        options: options,
        itemHeight: itemHeight,
        itemBuilder: itemBuilder,
        showBar: showBar,
        cancelStyle: cancelStyle,
        cancelText: cancelText,
        confirmStyle: confirmStyle,
        confirmText: confirmText,
        height: height,
        barBuilder: barBuilder,
      );
    },
  );
}

class _SparkCommonPicker<T> extends StatelessWidget {
  _SparkCommonPicker({
    required this.options,
    required this.dialogContext,
    this.height = 300,
    this.itemHeight = 40,
    this.itemBuilder,
    this.showBar = true,
    this.cancelText = '取消',
    this.confirmText = '确定',
    this.cancelStyle,
    this.confirmStyle,
    this.barBuilder,
    Key? key,
  }) : super(key: key);

  final BuildContext dialogContext;
  final List<SparkPickerData<T>> options;
  final double itemHeight;
  final SparkPickerBuilder? itemBuilder;
  final bool showBar;
  final String cancelText;
  final String confirmText;
  final TextStyle? cancelStyle;
  final TextStyle? confirmStyle;
  final double height;
  final SparkPickerTitleBarBuilder? barBuilder;

  final FixedExtentScrollController _controller =
      FixedExtentScrollController(initialItem: 0);

  Widget _sparkPickerItemBuilder({
    required BuildContext context,
    required int index,
    required List<SparkPickerData> options,
    required double itemHeight,
  }) {
    return Container(
      height: itemHeight,
      alignment: Alignment.center,
      child: Text(
        options[index].label,
        style: const TextStyle(fontSize: 14, color: Color(0xFF333333)),
        textAlign: TextAlign.start,
      ),
    );
  }

  void onCancel() {
    Navigator.of(dialogContext).pop(null);
  }

  void onConfirm() {
    SparkPickerData<T> _pikeData = options[_controller.selectedItem];
    Navigator.of(dialogContext).pop(_pikeData.value);
  }

  Widget? _buildBar(BuildContext context) {
    if (!showBar) {
      return null;
    }
    if (barBuilder != null) {
      return barBuilder!(context, onCancel, onConfirm);
    }
    return Container(
      padding: const EdgeInsets.symmetric(horizontal: 14, vertical: 10),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          GestureDetector(
            child: Text(
              cancelText,
              style: cancelStyle ??
                  const TextStyle(
                    fontSize: 14,
                    color: Color(0xFF333333),
                  ),
            ),
            behavior: HitTestBehavior.opaque,
            onTap: onCancel,
          ),
          GestureDetector(
            child: Text(
              confirmText,
              style: confirmStyle ??
                  const TextStyle(
                    fontSize: 14,
                    color: Color(0xFF2878FF),
                  ),
            ),
            behavior: HitTestBehavior.opaque,
            onTap: onConfirm,
          )
        ],
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    Widget? _bar = _buildBar(context);
    return SizedBox(
      height: height,
      child: Column(
        children: [
          if (_bar != null) _bar,
          Expanded(
            child: SparkPicker(
              controller: _controller,
              itemHeight: itemHeight,
              itemBuilder: itemBuilder ??
                  (_, _index) => _sparkPickerItemBuilder(
                        context: _,
                        itemHeight: itemHeight,
                        options: options,
                        index: _index,
                      ),
              itemCount: options.length,
            ),
          )
        ],
      ),
    );
  }
}

class SparkPicker extends StatelessWidget {
  const SparkPicker({
    Key? key,
    required this.controller,
    required this.itemHeight,
    required this.itemBuilder,
    required this.itemCount,
    this.onChanged,
    this.onChange,
    this.onStart,
    this.selectionOverlay,
  }) : super(key: key);

  /// picker controller
  final FixedExtentScrollController controller;

  /// single option height
  final double itemHeight;

  /// pick option builder
  final SparkPickerBuilder itemBuilder;

  /// item count
  final int itemCount;

  /// on changed, get last result
  final SparkPickerChanged? onChanged;

  /// scroll animation change index, get scrolling index value
  final SparkPickerChanged? onChange;

  final Function? onStart;

  /// selection overlay widget
  final Widget? selectionOverlay;

  void _selectedChangedWhenScrollEnd(int index) {
    if (onChanged != null) {
      onChanged!(index);
    }
  }

  void _selectedChangedWhenScrolling(int index) {
    if (onChange != null) {
      onChange!(index);
    }
  }

  @override
  Widget build(BuildContext context) {
    return NotificationListener(
      onNotification: (ScrollNotification notification) {
        if (notification.depth == 0 &&
            notification is ScrollEndNotification &&
            notification.metrics is FixedExtentMetrics) {
          final FixedExtentMetrics metrics =
              notification.metrics as FixedExtentMetrics;
          final int currentItemIndex = metrics.itemIndex;
          _selectedChangedWhenScrollEnd(currentItemIndex);
        }
        if (notification is ScrollStartNotification) {
          if (onStart != null) {
            onStart!();
          }
        }
        return false;
      },
      child: CupertinoPicker.builder(
        itemExtent: itemHeight,
        useMagnifier: true,
        selectionOverlay: selectionOverlay ??
            Container(
              decoration: const BoxDecoration(
                border: Border.symmetric(
                  horizontal: BorderSide(
                    color: Color(0xFFE5E5E5),
                  ),
                ),
              ),
            ),
        scrollController: controller,
        childCount: itemCount,
        itemBuilder: itemBuilder,
        onSelectedItemChanged: (int value) {
          _selectedChangedWhenScrolling(value);
        },
      ),
    );
  }
}

class SparkPickerData<T> {
  SparkPickerData.form(
    this.label,
    this.value,
    this.index,
  );

  /// show label
  String label;

  /// value
  T value;

  /// option index
  /// correspondence with the array
  int index;

  @override
  bool operator ==(Object other) {
    if (other is SparkPickerData) {
      return other.value == value && other.index == index;
    }
    return false;
  }

  @override
  int get hashCode => Object.hash(value, index, label);
}
